<?php

namespace App\Http\Controllers;

use App\Jobs\SyncUsersJob;
use App\Models\User;
use App\Models\UserGroup;
use App\Models\UserHasGroup;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use Spatie\Permission\Models\Role;
use DataTables;

class UserController extends Controller
{
    public function datatables($request, $records, $trashed = null)
    {
        if (!empty($request->search['value'])) {
            $searchValue = $request->search['value'];
            $records = $records->where(function ($query) use ($searchValue) {
                $query->where('name', 'LIKE', "%$searchValue%")
                    ->orWhere('email', 'LIKE', "%$searchValue%");
                $query->orWhereHas('role', function ($query) use ($searchValue) {
                    $query->where('name', 'LIKE', "%$searchValue%");
                });
            });
        }
        $totalRecords = $records->count(); // Get the total number of records for pagination
        $data = $records->skip($request->start)
            ->take($request->length)
            ->get();
        return DataTables::of($data)
            ->addIndexColumn()
            ->editColumn('sr_no', function ($row) {
                return $row->id;
            })
            ->addColumn('checkbox', function ($row) {
                return '<input type="checkbox" data-user-id="'.$row->id.'" class="user-checkbox">';
            })
            ->addColumn('name', function ($row) {
                return ucwords($row->name);
            })
            ->addColumn('email', function ($row) {
                return $row->email;
            })
            ->addColumn('role', function ($row) {
                return $row->role?->name;
            })
            ->addColumn('group', function ($row) {
                $row->load('groups.group');

                $groupNames = $row->groups->map(function ($userHasGroup) {
                    return $userHasGroup->group?->name;
                })->toArray();

                return implode(', ', $groupNames);
            })
            ->addColumn('actions', function ($row) use ($trashed) {
                $btns = '
                    <div class="actionb-btns-menu d-flex justify-content-center">';
                    if ($trashed == null) {
                        $btns .= '<a class="btn btns m-0 p-1" data-user-id="'.$row->id.'" href="users/edit/'.$row->id.'">
                                <i class="align-middle text-primary" data-feather="edit">
                                </i>
                            </a>
                            <a class="btn btns m-0 p-1 delete-user" data-user-id="'.$row->id.'" href="#">
                                <i class="align-middle text-danger" data-feather="trash-2">
                                </i>
                            </a>
                        </div>';
                    } else {
                        $btns.= '<a class="btn btns m-0 p-1" href="restore/' . $row->id . '">
                                <i class="align-middle text-success" data-feather="refresh-cw">
                                </i>
                            </a>
                            <a class="btn btns m-0 p-1" href="#">
                                <i class="align-middle text-danger" data-feather="trash-2">
                                </i>
                            </a>
                        </div>';
                    }
                return $btns;
            })
            ->rawColumns(['sr_no', 'checkbox', 'name', 'role', 'group', 'actions'])
            ->setTotalRecords($totalRecords)
            ->setFilteredRecords($totalRecords) // For simplicity, same as totalRecords
            ->skipPaging()
            ->make(true);
    }
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        //
        if ($request->ajax()) {
            $records = User::with('groups', 'role')->orderBy('id', 'asc');
            return $this->datatables($request, $records);
        }
        $data['header_title'] = 'Users List';
        return view('admin.users.index', $data);
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        //
        $data['header_title'] = 'Add New';
        $data['roles'] = Role::get();
        $data['groups'] = UserGroup::get();
        return view('admin.users.create', $data);
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        //
        request()->validate([
            'name' => 'required|string',
            'email' => 'required|email|unique:users',
            'password' => 'required|string',
            'role_id' => 'required'
        ]);
        if (!empty($request->group_id)) {
            $user = new User();
            $user->name = $request->name;
            $user->email = $request->email;
            $user->password =  Hash::make($request->password);
            $user->user_type = $request->role_id;
            $user->save();
            $role = Role::find($request->role_id);
            $user->assignRole($role);
            $groupIds = $request->group_id;
            foreach ($groupIds as $groupId) {
                $userHasGroups = new UserHasGroup();
                $userHasGroups->user_id = $user->id;
                $userHasGroups->group_id = $groupId;
                $userHasGroups->save();
            }
            return redirect('admin/users')->with('success', 'User added successfully');
        }
    }

    /**
     * Display the specified resource.
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($id)
    {
        //
        $data['header_title'] = 'Edit User Details';
        $data['roles'] = Role::get();
        $data['groups'] = UserGroup::get();
        $data['record'] = User::find($id);
        return view('admin.users.edit', $data);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(Request $request, $id)
    {
        //
        try {
            request()->validate([
                'name' => 'required|string',
                'email' => 'required|email|unique:users,email,' . $id,
                'role_id' => 'required'
            ]);
            $user = User::find($id);
            $user->name = $request->name;
            $user->email = $request->email;
            if (!empty($request->password)) {
                $user->password =  Hash::make($request->password);
            }
            $user->user_type = $request->role_id;
            $user->save();
            $role = Role::find($request->role_id);
            $user->assignRole($role);
            if (!empty($request->group_id)) {
                $groupIds = $request->group_id;
                UserHasGroup::where('user_id', $user->id)->delete();
                foreach ($groupIds as $groupId) {
                    $userHasGroups = new UserHasGroup();
                    $userHasGroups->user_id = $user->id;
                    $userHasGroups->group_id = $groupId;
                    $userHasGroups->save();
                }
            }
            return redirect('admin/users')->with('success', 'User updated successfully');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(Request $request)
    {
        if (!empty($request->user_id)) {
            if ($request->user_id == 1) {
                return response()->json(['status' => false, 'message' => 'Cannot delete admin user']);
            }
            User::where('id', $request->user_id)->where('id', '!=', 1)->delete();
            return response()->json(['status' => true, 'message' => 'Deleted successfully']);
        } else {
            return response()->json(['status' => false,'message' => 'User not found']);
        }
    }

    public function sync()
    {
        try {
            dispatch(new SyncUsersJob());
            return redirect()->back()->with('success', 'Synchronization started successfully');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', 'Something went wrong');
        }
    }

    public function deleteMultipleUsers(Request $request)
    {
        $userIds = $request->user_ids;
        if (!empty($userIds)) {
            User::whereIn('id', $userIds)->where('id', '!=', 1)->delete();
            return response()->json(['status' => true, 'message' => 'Deleted successfully']);
        }
    }

    public function trashed(Request $request)
    {
        if ($request->ajax()) {
            $records = User::with('groups', 'role')->onlyTrashed()->orderBy('id', 'asc');
            return $this->datatables($request, $records, 'trashed');
        }
        $data['header_title'] = 'Trashed Users List';
        return view('admin.users.trashed', $data);
    }

    public function restore(Request $request, $userId)
    {
        if (!empty($userId)) {
            User::onlyTrashed()->whereIn('id', [$userId])->restore();
            return redirect()->back()->with('success', 'User restored successfully');
        } else {
            return redirect()->back()->with('error', 'User not found');
        }
    }
}
